import math
from enum import Enum
import time, sys
from collections.abc import Iterable


class log_level_enum(Enum):
    """
    log级别枚举
    """
    Error = 1
    Warning = 2
    normal = 3
    pigeonhole = 4


def log(str, log_level: log_level_enum = log_level_enum.normal):
    """
    打印log
    :param str: log内容
    :param log_level: log级别，使用枚举
    :return: 无返回
    """
    print(f"[{log_level}] {str} \n")


def log_error(error_message: str):
    """
    输出错误log，并抛出异常
    :param error_message: 异常信息
    :return: 无返回
    """
    log(error_message, log_level_enum.Error)
    raise Exception(error_message)


current_process_bar = None


class process_status_bar:
    """
    控制台的进度条
    """

    def __init__(self, processbar_length=20):
        """
        初始化函数
        :param processbar_length: 进度条长度（打印长度）
        """
        self._iter_stack = []
        self._process_bar_len = processbar_length
        self._print_str = ""
        self._print_logs = []
        global current_process_bar
        current_process_bar = self

    def _cast_second_strformat(self, sec):
        m, s = divmod(sec, 60)
        h, m = divmod(m, 60)
        return "%02d:%02d:%02d" % (h, m, s)

    def _flush_(self):
        stack_str = ""
        for iter_i in range(len(self._iter_stack) - 1):
            p_value = self._iter_stack[iter_i]['value'] / float(self._iter_stack[iter_i]['max'])
            p_value = "%.2f" % (p_value * 100)
            p_count = f"{self._iter_stack[iter_i]['value']}/{self._iter_stack[iter_i]['max']}"
            stack_str += f"[{self._iter_stack[iter_i]['key']}  {p_count}  {p_value}%] >>"
            pass

        current_item = self._iter_stack[-1]
        p_value = 0 if float(current_item["max"]) == 0 else current_item["value"] / float(current_item["max"])
        p_value_str = "%.2f" % (p_value * 100)

        bar_size = math.floor(p_value * self._process_bar_len)

        bar_str = ""
        for i in range(self._process_bar_len):
            if i + 1 <= bar_size:
                bar_str += "*"
            else:
                bar_str += " "

        if current_item['count'] != 0:
            avg_cost = current_item['total_cost'] / current_item['count']
            sur_plus = avg_cost * (current_item['max'] - current_item['value'])
            time_show = f"{self._cast_second_strformat(avg_cost)},{self._cast_second_strformat(sur_plus)}"
        else:
            time_show = "00:00:00"

        bar_str = f'{stack_str} {current_item["key"]} {current_item["value"]}/{current_item["max"]}  [{bar_str}]{p_value_str}% [{time_show}] - {self._print_str} '

        if len(self._print_logs) == 0:
            sys.stdout.write('\r' + bar_str)
        else:
            sys.stdout.write('\r')
            for s in self._print_logs:
                sys.stdout.write(s + "\n")
            self._print_logs.clear()
            sys.stdout.write(bar_str)
        sys.stdout.flush()

    def iter_bar(self, iter_item, value=0, key=None, max=None):
        """
        进度条的遍历方法
        :param iter_item: 主遍历对象
        :param value: value起始
        :param key: 遍历名称
        :param max: 遍历的数量
        :return: 可遍历对象
        """

        def get_length(iter_item):
            if isinstance(iter_item, Iterable):
                try:
                    return len(iter_item)
                except TypeError:
                    # 'iter_item' is an iterable but doesn't have a __len__ method.
                    # It could be something like an infinite generator.
                    log_error("该对象为无限长度的可迭代对象")
            else:
                log_error("需要指定max值")

        if max is None:
            max = get_length(iter_item)

        if key is None:
            key = f"Iter {len(self._iter_stack)}"
        self._iter_stack.append({
            'key': key,
            'value': value,
            'max': max,
            'total_cost': 0,
            'count': 0,
        })

        time_stamp = time.time()
        for _iter in iter_item:
            self._flush_()
            yield _iter
            self._iter_stack[-1]['value'] += 1
            self._iter_stack[-1]['count'] += 1
            self._iter_stack[-1]['total_cost'] += time.time() - time_stamp
            time_stamp = time.time()

        self._flush_()
        self._iter_stack.pop(-1)

    def update_process_value(self, v):
        """
        强制更新进度条的进度数值
        :param v: 要更新的数值
        :return: 无返回
        """
        self._iter_stack[-1]['value'] = v

    def process_print(self, str):
        """
        打印过程中的状态信息（覆盖输出）
        :param str: 状态信息
        :return: 无返回
        """
        self._print_str = str

    def print_log(self, str):
        """
        过程中打印log（不覆盖，使用换行）
        :param str: 状态信息
        :return: 无返回
        """
        self._print_logs.append(str)


def process_bar_iter(iter_item, key=None, max=None):
    """
    迭代过程的简易进度条
    :param iter_item: 迭代对象
    :param key: 迭代名称
    :param max: 最大迭代数量
    :return: 无返回
    """
    pb = process_status_bar()
    for item in pb.iter_bar(iter_item, key=key, max=max):
        yield item

# psb = process_status_bar()
# for s1 in psb.iter_bar([1, 2, 3], "key1", max=3):
#     for s2 in psb.iter_bar([1, 2, 3], "key2", max=3):
#         for s2 in psb.iter_bar([1, 2, 3], "key3", max=3):
#             time.sleep(0.3)
